//"Fabulous Fred" Game
//For Bolt 18F2550 system with LCD and keypad
//Uses 1 second interrupts with timer 0
//Creates true random numbers to generate
//new combinations for the game. 
//Microswitches must have the value of level 
//SW1=OFF =>Easy
//SW1=OFF,SW2=OFF => Hard
//SW1=OFF,SW2=OFF,SW3=OFF => Expert
//The system shows first a sequence in leds.
//User must then type in keypad the number of each  
//of the activated leds (0...7).
//Author: Valencia Molina Nydia Angelica

#include <p18cxxx.h>
#include <delays.h>
#include <stdlib.h>
#include "18F2550BOLT.h"
#include <timers.h>
#define CLOCK_TIMER0 12000000;

void timer_isr(void);
extern void _startup( void ); 

#pragma code _RESET_INTERRUPT_VECTOR = 0x000800 
void _reset( void ) 
{ 
    _asm goto _startup _endasm 
} 

#pragma code _HIGH_INTERRUPT_VECTOR = 0x000808
void _high_ISR (void)
{
	_asm goto timer_isr _endasm
}


#pragma code

const char inicio[]="WELCOME";
const char nivel[]="Select level";
const char level[]="Level";
const char facil[]="EASY";
const char dificil[]="HARD";
const char experto[]="EXPERT";
const char falla[]="ERROR";
const char juego[]="Starts Game";
const char round[]="ROUND ";
const char ganaste[]="WIN!! :)";
const char perdiste[]="LOST :(";
const char nuevo[]="New Game";
const char end[]="END...";
long int semilla=342; //semilla inicial para la generacin de un nmero aleatorio
int numLeds; //nmero de leds a encenderse en la ronda
int numRonda; //nmero de ronda actual
int retraso; //retraso entre encendido de leds en la ronda
int dec; //decremento en el retraso para la siguiente ronda
int i; //auxiliar
int leds[8]; //arreglo que registra el orden y nmero de los leds encendidos en la ronda
int usuario[8]; //arreglo que registra el orden y el nmero de leds seleccionado por el usuario
char array[8];
int long Ticker; 

#pragma interrupt timer_isr
void timer_isr(void)
{
  Ticker=Ticker-(65536*2);                    // Decrementa ticker  
  if (Ticker<65536)                      // si ya transcurri medio segundo 
  {
		Ticker=Ticker+CLOCK_TIMER0;      	//   Incrementa ticker 
    	semilla+=1;                           //   Incrementa numbero de segundos 
		if(RELE==0) RELE =1;
		else RELE=0;
		delay_ms(50);
  } 
  INTCONbits.TMR0IF = 0 ;	
}

void start();
void ronda();
void ledsAleatorio();
void error();
void final();

void main()
{
		init_bolt();
		InitLCD();
		Ticker=CLOCK_TIMER0;
		OpenTimer0 (TIMER_INT_ON & T0_SOURCE_INT & T0_16BIT);
		INTCONbits.GIE = 1;
		start();
} 

void start()//funcin para obtener los parmetros iniciales del juego
{
		int n;
		ClearScreen();
		PrintString(inicio);//texto de bienvenida
		delay_ms(1000);
		ClearScreen();
		PrintString(nivel);//texto de nivel elegido
		GotoXY(1,0);
		for(i=8; i<13; i++)
		{
			PutLCD(nivel[i]);
		}
		for(i=9;i>=0;i--) //Tiempo para que el usuario elija nivel
		{
			GotoXY(1,6);
			PutLCD('>');
			PutLCD(i+0x30);
			delay_ms(1000);
		}

		n = SW1+SW2*2+SW3*4;//eleccin de nivel

		ClearScreen();
							//conjunto de valores iniciales para cada nivel
		PrintString(level);
		GotoXY(1,0);
		if(n==1)//nivel fcil
		{
			PrintString(facil);
			numLeds=3;
			retraso=2000;
			dec = 100;
		}
		else if (n==3)//nivel difcil
		{
			PrintString(dificil);
			numLeds=4;
			retraso=1200;
			dec=100;
		}
		else if(n==7)//nivel esperto
		{
			PrintString(experto);
			numLeds=5;
			retraso=400;
			dec=50;
		}
		else//error
		{
			error();
		}
		delay_ms(1000);
		ClearScreen();
		PrintString(juego);//inicia juego 
		GotoXY(1,0);
		for(i=8; i<12; i++)
		{
			PutLCD(juego[i]);
		}
		delay_ms(3000);
		ronda();
		return;
}

void ronda()//gestin del juego
{
	for(numRonda=1; numRonda<9; numRonda++)
	{
		ClearScreen();
		PORTB=0X00;
		PrintString(round);
		PutLCD(numRonda+0x30);
		srand(semilla+numRonda+dec);
		ledsAleatorio();
		GotoXY(0,7);
		PutLCD('>');
		GotoXY(1,0);
		for(i=0;i<numLeds;i++)
		{
			usuario[i]=GetBot();
			PutLCD(usuario[i]);
			usuario[i]-=0x30;
			if(usuario[i]!=leds[i]) //el usuario se equivoca 
			{
				ClearScreen();
				PrintString(perdiste);
				delay_ms(2000);
				final();
			}
		}
		retraso-=dec;
		if((numRonda==3)||(numRonda==6)) numLeds++;
		
	}
	ClearScreen();
	PrintString(ganaste);
	RELE=1;
	for(i=0;i<5;i++)
	{
		delay_ms(1000);
		if (PORTB==0x00)PORTB=0xFF;
		else PORTB=0x00;
	}
	final();
	return;
}
void ledsAleatorio()//funcin para obtener los leds aleatorios que se encendern en cada ronda
{
	int j;
	for(j=0; j<numLeds;j++)
	{
		leds[j] = rand()%8; //0-32767, rand() % (N+1), donde n es el nmero mximo que se desea obtener
		PORTB=0x01;
		PORTB=PORTB<<leds[j];
		delay_ms(retraso);
	}
	return;
	
}

void error()//el usuario cometi algn error durante la ejecucin del programa
{
	ClearScreen();
	PrintString(falla);
	delay_ms(2000);
	final();	
	return;
}

void final()//fin de juego
{
	ClearScreen();
	PrintString(nuevo);
	GotoXY(1,0);
	PutLCD('?');
	PutLCD(' ');
	for(i=9;i>=0;i--) //Tiempo para que el usuario elija nivel
	{
		GotoXY(1,2);
		PutLCD('>');
		PutLCD(i+0x30);
		delay_ms(1000);
	}
	if(SW1==1)//comenzar un nuevo juego
	{
		PORTB=0x00;
		RELE=0;
		start();
	}
	else//ciclo infinito
	{
		ClearScreen();
		PrintString(end);
		while(1);
	}
	return;
}